/** \file apop_output.c	 Some printing and output interface functions. 

Copyright (c) 2006--2007 by Ben Klemens.  Licensed under the modified GNU GPL v2; see COPYING and COPYING2.  */

#include "output.h"
#include "settings.h"
#include "bootstrap.h"
#include "conversions.h"
#include "model/model.h"
#include <gsl/gsl_histogram.h>
/** Prep for gnuplot one of those cute scatterplots with a regression line through it.

Currently, you only get two dimensions.

Set the global \ref apop_opts_type "apop_opts.output_name" to the filename you want before running this.
It appends instead of overwriting, so you can prep the file if you want; see sample code. [to overwrite a file, just remove it first with the standard C function <tt>remove("filename");</tt>]


\param	data	This is a copy of what you'd sent to the regression fn. That is, the first column is the dependent variable and the second is the independent. That is, what will be on Y axis is the <i>first</i> column, and what is on the X axis is the second. Custom for regressions and custom for graphs just clash on this one.
\param	est	The \ref apop_model structure your regression function gave you.
\param outfile The name of the output file. NULL will send output to STDOUT.

The sample program below will pull data from a database (you'll need to
modify it to produce your own two-column table), then runs OLS, produces
a gnuplot file to write to a file named "scatter.eps", and finally calls
gnuplot for you. You'll still have the gnuplot file ("auto") if you want
to make further modifications.

\code
int main(){
apop_data 	    *data, *data_copy;
apop_model   *est;
FILE		    *f;
char		    outfile[]	= "auto",
		        do_me[10000];

	apop_db_open("cpia.db");
	data      =apop_query_to_data("select gnppercap, cpia from cpiagnp;");
	apop_close_db(0);

	//The regression destroys your data, so copy it first.
	data_copy   = apop_data_copy(data);

	//Run OLS, display results on terminal
	est  = apop_estimate(data, apop_OLS);
	apop_estimate_print(est);

	//Prep the file with a header, then call the function.
	f    = fopen(outfile, "w");
	fprintf(f,"set term postscript;\n set output \"scatter.eps\"\n set yrange [0:*]\n");
	fclose(f);
	apop_plot_line_and_scatter(data_copy,est, outfile);

	//Have the machine run gnuplot for you. 
	sprintf(do_me, "gnuplot -persist %s", outfile);
	system (do_me);
	return 0;
}
\endcode
\todo The sample data here should correspond to that which apophenia ships with.
\ingroup output
*/
void apop_plot_line_and_scatter(apop_data *data, apop_model *est, char * outfile){
  FILE  *f;
  char  exdelimiter[100];
  int   append_state;
    f   = (outfile? fopen(outfile, "a") : stdout);
	fprintf(f, "f(x) = %g  + %g * x\n", gsl_vector_get(est->parameters->vector,0), gsl_vector_get(est->parameters->vector,1));
	if (data->names){
		fprintf(f, "set xlabel \"%s\"\n", data->names->column[1]);
        if (est->expected !=NULL)
		    fprintf(f, "set ylabel \"%s\"\n", est->expected->names->column[0]);
	}
	fprintf(f, "plot \"-\" using 2:1 , f(x) with lines;\n");
    if (apop_opts.output_type != 0) 	fclose(f);

    //force the delimiter to be a comma space; don't tell the user.
    strcpy(exdelimiter, apop_opts.output_delimiter);
    strcpy(apop_opts.output_delimiter, ", ");
    append_state            = apop_opts.output_append;
    apop_opts.output_append = 1;
	apop_matrix_print(data->matrix, outfile);
    strcpy(apop_opts.output_delimiter, exdelimiter);
    apop_opts.output_append = append_state;
}

/** This function can be used to temporarily modify the global options,
 to facilitate better encapsulation of code. Usage:

  \code
  apop_opts_type tmp_opts;
  apop_opts_memcpy(&tmp_opts, &apop_opts);
  strcpy(apop_opts.output_name, "ad_hoc_temp_file");
  [do things here]
  apop_opts_memcpy(&apop_opts, &tmp_opts);
  \endcode

If you just need a little more verbosity for a procedure, you probably
don't need to use this function. Just try: 
  \code
  apop_opts.verbose ++;
  [do things here]
  apop_opts.verbose --;
  \endcode

The philosophy is that the global variables are generally not going
to change over the course of a program: either you are working on the
screen, in the database, or piping out of STDOUT, and you likely won't
change mid-stream. Thus, it is easier to set these globally at the top of
the program but less convenient to switch frequently throughout the code.
\ingroup global_vars
 */
void apop_opts_memcpy(apop_opts_type *out, apop_opts_type *in){
    memcpy(out, in, sizeof(apop_opts_type));
}


/** This function will plot a histogram model. Compare with \ref apop_plot_histogram, which is a convenience function to plot a \c gsl_vector as a histogram.

The function respects the <tt>output_type</tt> option, so code like:
\code
apop_opts.output_type = 'p';
apop_opts.output_pipe = popen("/usr/bin/gnuplot", "w");
apop_model *m = apop_estimate(data, apop_histogram);
apop_plot_histogram(m);
\endcode
will print directly to Gnuplot.

\param hist A parametrized \ref apop_model holding the histogram.  (No default. Must not be \c NULL.)
\param outfile  The file to be written. If NULL then write to STDOUT. (Default = \c NULL.)

This function uses the \ref designated syntax for inputs.
  \ingroup output
*/
#ifdef APOP_NO_VARIADIC 
 void apop_histogram_plot(apop_model *hist, char *outfile){
#else
apop_varad_head( void , apop_histogram_plot){
      apop_model * apop_varad_var(hist, NULL);
      apop_assert_void(hist, 0, 's', "Input histogram is NULL.\n");
      char * apop_varad_var(outfile, NULL);
      apop_histogram_plot_base(hist, outfile);
}

 void apop_histogram_plot_base(apop_model *hist, char *outfile){
#endif
  int             i;
  FILE           *f;
  gsl_histogram  *h  = Apop_settings_get(hist, apop_histogram, pdf);
    if (!h) h = Apop_settings_get(hist, apop_kernel_density, pdf);

	//Now that you have a histogram, print it.
    if (apop_opts.output_type == 'p')
        f   = apop_opts.output_pipe;
    else 
        f = (outfile ? fopen(outfile, "a") : stdout);
	fprintf(f, "set key off					                    ;\n\
                        plot '-' with lines\n");
    //I can't tell which versions of Gnuplot support this form:
	/*fprintf(f, "set key off					                    ;\n\
                        set style data histograms		        ;\n\
                        set style histogram cluster gap 0	    ;\n\
                        set xrange [0:%i]			            ;\n\
                        set style fill solid border -1          ;\n\
                        set boxwidth 0.9                        ;\n\
                        plot '-' using 2:xticlabels(1);\n", bins);*/
	for (i=0; i < h->n-1; i++)
	    fprintf(f, "%4f\t %g\n", h->range[i], gsl_histogram_get(h, i));
	fprintf(f, "e\n");
    if (apop_opts.output_type == 'p')
        fflush(f);
    else if (outfile)    
        fclose(f);
}



/** This function will take in a \c gsl_vector of data and put out a histogram. Compare with \ref apop_histogram_plot, which plots an estimated \ref apop_histogram model.

The function respects the <tt>output_type</tt> option, so code like:
\code
f   = popen("/usr/bin/gnuplot", "w");
apop_opts.output_type = 'p';
apop_opts.output_pipe = f;
apop_plot_histogram(data, 100, NULL);
\endcode
will print directly to Gnuplot.

\param data A \c gsl_vector holding the data. Do not pre-sort or bin; this function does that for you. (no default, must not be \c NULL)
\param bins   The number of bins in the output histogram (default = MAX(10, data->size/20); denominator subject to future adjustment)
\param outfile  The file to be written. If NULL then write to STDOUT. (default = \c NULL)

This function uses the \ref designated syntax for inputs.
  \ingroup output
*/
#ifdef APOP_NO_VARIADIC 
 void apop_plot_histogram(gsl_vector *data, size_t bins, char *outfile){
#else
apop_varad_head( void , apop_plot_histogram){
      gsl_vector * apop_varad_var(data, NULL);
      apop_assert_void(data, 0, 's', "Input histogram is NULL.\n");
      char * apop_varad_var(outfile, NULL);
      size_t apop_varad_var(bins, GSL_MAX(10, data->size/20));
      apop_plot_histogram_base(data, bins, outfile);
}

 void apop_plot_histogram_base(gsl_vector *data, size_t bins, char *outfile){
#endif
  int             i;
  FILE *          f;
  double          min=GSL_POSINF, max=GSL_NEGINF, pt;
  gsl_histogram   *h      = gsl_histogram_alloc(bins);
    gsl_vector_minmax(data, &min, &max);
        gsl_histogram_set_ranges_uniform(h, min-GSL_DBL_EPSILON, max+GSL_DBL_EPSILON);
    for (i=0; i < data->size; i++){
        pt  = gsl_vector_get(data, i);
        if (!gsl_isnan(pt))
           gsl_histogram_increment(h, pt);
        }
    //Now that you have a histogram, print it.
    if (apop_opts.output_type == 'p')
        f   = apop_opts.output_pipe;
    else
        f = (outfile ? fopen(outfile, "a") : stdout);
    fprintf(f, "set key off                                     ;\n\
                        plot '-' with lines\n");
    //I can't tell which versions of Gnuplot support this form:
    /*fprintf(f, "set key off                                       ;\n\
                        set style data histograms               ;\n\
                        set style histogram cluster gap 0       ;\n\
                        set xrange [0:%i]                       ;\n\
                        set style fill solid border -1          ;\n\
                        set boxwidth 0.9                        ;\n\
                        plot '-' using 2:xticlabels(1);\n", bins);*/
    for (i=0; i < bins; i++)
        fprintf(f, "%4f\t %g\n", h->range[i], gsl_histogram_get(h, i));
    fprintf(f, "e\n");
    if (apop_opts.output_type == 'p')
        fflush(f);
    else if (outfile)
        fclose(f);
}


	
/** Print an \c apop_histogram. Put a "plot '-'\n" before this, and
 you can send it straight to Gnuplot. The -inf and +inf elements are not printed. 
 
\param h The input histogram (no default, must not be \c NULL.)
\param outfile  The file to be written. If NULL then write to STDOUT. (default = \c NULL)
 
This function uses the \ref designated syntax for inputs.
 */
#ifdef APOP_NO_VARIADIC 
 void apop_histogram_print(apop_model *h, char *outfile){
#else
apop_varad_head( void , apop_histogram_print){
      apop_model * apop_varad_var(h, NULL);
      apop_assert_void(h, 0, 's', "Input histogram is NULL.\n");
      char * apop_varad_var(outfile, NULL);
      apop_histogram_plot_base(h, outfile);
}

 void apop_histogram_print_base(apop_model *h, char *outfile){
#endif
  apop_histogram_settings *hp = apop_settings_get_group(h, "apop_histogram"); 
  if (!hp) hp = apop_settings_get_group(h, "apop_kernel_density"); 
  if (!hp)
      apop_error(0, 's', "%s: You sent me an apop_model with no histogram settings. Have you estimated this histogram with data yet?\n", __func__);
  int             i;
  FILE *          f;
    if (apop_opts.output_type == 'p')
        f   = apop_opts.output_pipe;
    else 
        f = (outfile ? fopen(outfile, "a") : stdout);
	for (i=1; i < hp->pdf->n-1; i++)
	    fprintf(f, "%4f\t %g\n", hp->pdf->range[i], gsl_histogram_get(hp->pdf, i));
    if (apop_opts.output_type == 'p')
        fflush(f);
    else if (outfile)    
        fclose(f);
}

////////////////////////////
/////The printing functions.
////////////////////////////

void white_pad(int ct){
  size_t i;
    for(i=0; i < ct; i ++)
        printf(" ");
}

/** This function prettyprints the apop_data set to a screen.

This takes a lot of machinery. I write every last element to a text
array, then measure column widths, then print to screen with padding to
guarantee that everything lines up.  There's no way to have the first
element of a column line up with the last unless you interrogate the
width of every element in the column, so printing columns really can't be a one-pass 
process.

So, I produce an \ref apop_data set with no numeric elements and a text
element to be filled with the input data set, and then print that. That
means that I'll be using (more than) twice the memory to print this. If
this is a problem, you can use \ref apop_print to dump your data to a text file,
and view the text file, or print subsets.

For more machine-readable printing, see \ref apop_print.

\ingroup output

*/
void apop_data_show(const apop_data *in){
  size_t i, j;
//Take inventory and get sizes
  size_t hasrownames = in->names->rowct ? 1 : 0;
  size_t hascolnames = (in->names->vector || in->names->colct || in->names->textct);
  size_t hasvector   = in->vector ? 1 : 0;
  size_t matrixcols  = in->matrix ? in->matrix->size2 : 0;

    size_t outsize_r = GSL_MAX(in->matrix ? in->matrix->size1 : 0, in->vector ? in->vector->size: 0);
    outsize_r   = GSL_MAX(outsize_r, in->textsize[0]);
    outsize_r   += hascolnames;

    size_t outsize_c = matrixcols;
    outsize_c   += in->textsize[1];
    outsize_c   += hasvector;
    outsize_c   += hasrownames;

//Write to the printout data set.
    apop_data *printout = apop_text_alloc(NULL , outsize_r, outsize_c);
    for(i= 0; i< outsize_r; i ++)
        for(j= 0; j < outsize_c; j ++)
            apop_text_add(printout, i, j, "");

    if (hasrownames)
        for(i=0; i < in->names->rowct; i ++)
            apop_text_add(printout, i + hascolnames, 0, in->names->row[i]);
    if (hasvector)
        for(i=0; i < in->vector->size; i ++)
            apop_text_add(printout, i + hascolnames, hasrownames, "%g", gsl_vector_get(in->vector, i));
    if (matrixcols)
        for(i=0; i < in->matrix->size1; i ++)
            for(j=0; j < in->matrix->size2; j ++)
                apop_text_add(printout, i + hascolnames, hasrownames + hasvector+ j, "%g", gsl_matrix_get(in->matrix, i, j));
    if (in->textsize[0])
        for(i=0; i < in->textsize[0]; i ++)
            for(j=0; j < in->textsize[1]; j ++)
                apop_text_add(printout, i + hascolnames, hasrownames + hasvector+ matrixcols + j, in->text[i][j]);

//column names
    if (hascolnames){
        if (hasvector && in->names->vector)
                apop_text_add(printout, 0 , hasrownames,  in->names->vector);
        if (matrixcols)
            for(i=0; i < in->names->colct; i ++)
                apop_text_add(printout, 0 , hasrownames + hasvector + i,  in->names->column[i]);
        if (in->textsize[1])
            for(i=0; i < in->names->textct; i ++)
                apop_text_add(printout, 0 , hasrownames + hasvector + matrixcols + i, in->names->text[i]);
    }

//get column sizes
    int colsizes[outsize_c];
    for(i=0; i < outsize_c; i ++){
        colsizes[i] = strlen(printout->text[0][i]);
        for(j=1; j < outsize_r; j ++)
            colsizes[i] = GSL_MAX(colsizes[i], strlen(printout->text[j][i]));
    }

//Finally, print
    for(j=0; j < outsize_r; j ++){
        for(i=0; i < outsize_c; i ++){
            white_pad(colsizes[i] - strlen(printout->text[j][i]) + 1);//one spare space.
            printf(printout->text[j][i]);
            if (i > 0 && i< outsize_c-1) 
                printf(" %s ", apop_opts.output_delimiter);
        }
        printf("\n");
    }

    apop_data_free(printout);
    return;
}






static void print_core_v(const gsl_vector *data, char *separator, char *filename, 
			void (* p_fn)(FILE * f, double number)){
int 		i;
FILE * 		f;
	if (!filename || !strcmp(filename, "STDOUT"))
		f	= stdout;
	else	
        f = fopen(filename, apop_opts.output_append ? "a": "w");
    if (apop_opts.output_type == 'p')
        f   = apop_opts.output_pipe;
    if (!data)
        fprintf(f, "NULL\n");
    else {
	    for (i=0; i<data->size; i++){
		    p_fn(f, gsl_vector_get(data, i));
		    if (i< data->size -1)	fprintf(f, "%s", separator);
	    }
	    fprintf(f,"\n");
    }
	if (filename && apop_opts.output_type != 'p')	fclose(f);
}

static void print_core_m(const gsl_matrix *data, char *separator, char *filename, 
			void (* p_fn)(FILE * f, double number), apop_name *n){
  FILE * 	f;
  size_t    i,j; 
  int       max_name_size  = 0;
    if (n)
        for (i=0; i< n->rowct; i++)
            max_name_size   = GSL_MAX(strlen(n->row[i]), max_name_size);

    if (apop_opts.output_type == 'p'){
        f   = apop_opts.output_pipe;
        if (!f){ 
            apop_error(1, 'c', "%s: You set apop_opts.output_type to 'p', but apop_opts.output_pipe is NULL. Assuming you meant stdout.\n", __func__);
            f = stdout;
        }
    } else {
          if ((filename == NULL) || (!strcmp(filename, "STDOUT")))
            f	= stdout;
          else
            f	= fopen(filename, apop_opts.output_append ? "a" : "w");
    }
    if (n && strlen(n->title)>0)
        fprintf(f, "%s%s\n\n", apop_opts.output_type=='s'? "\t\t" : "", n->title);
    if (!data)
        fprintf(f, "NULL\n");
    else {
        if (n && n->colct > 0){ //then print a row of column headers.
		    fprintf(f,"\t");
		    for (j=0; j< n->colct; j++)
			    fprintf(f,"%s\t\t", n->column[j]);
		    fprintf(f,"\n");
        }
	    for (i=0; i<data->size1; i++){
            if (n && n->rowct > 0)
			    fprintf(f,"%-*s", max_name_size+4, n->row[i]);
		    for (j=0; j<data->size2; j++){
			    p_fn(f, gsl_matrix_get(data, i,j));
			    if (j< data->size2 -1)	fprintf(f, "%s", separator);
		    }
		    fprintf(f,"\n");
	    }
    }
	if (filename !=NULL && apop_opts.output_type != 'p')	fclose(f);
}

void dumb_little_pf(FILE * f, double data){
    if (data == (int) data)
	    fprintf(f, "% 5i", (int) data); 
    else
        fprintf(f, "% 5f", data);
}

/** Print a vector in float format.
    You may want to set \ref apop_opts_type "apop_opts.output_delimiter".
\ingroup apop_print */
void apop_vector_print(gsl_vector *data, char *file){
	print_core_v(data, apop_opts.output_delimiter, file, dumb_little_pf); }

/** Print a matrix in float format.
    You may want to set \ref apop_opts_type "apop_opts.output_delimiter".
\ingroup apop_print */
void apop_matrix_print(gsl_matrix *data, char *file){
    if (apop_opts.output_type   == 'd')
        apop_matrix_to_db(data, apop_strip_dots(apop_strip_dots(file,1),0), NULL);
    else
        print_core_m(data, apop_opts.output_delimiter, file, dumb_little_pf, NULL); 
}


/** Dump a <tt>gsl_vector</tt> to the screen. 
    You may want to set \ref apop_opts_type "apop_opts.output_delimiter".
\ingroup apop_print */
void apop_vector_show(const gsl_vector *data){
  char tmptype    = apop_opts.output_type;
    apop_opts.output_type = 's';
	print_core_v(data, apop_opts.output_delimiter, NULL, dumb_little_pf); 
    apop_opts.output_type = tmptype;
}

/** Dump a <tt>gsl_matrix</tt> to the screen.
    You may want to set \ref apop_opts_type "apop_opts.output_delimiter".
\ingroup apop_print */
void apop_matrix_show(const gsl_matrix *data){
  char tmptype    = apop_opts.output_type;
    apop_opts.output_type = 's';
	print_core_m(data, apop_opts.output_delimiter, NULL, dumb_little_pf, NULL); 
    apop_opts.output_type = tmptype;
}

static int get_max_strlen(char **names, size_t len){
  int   i, 
        max  = 0;
    for (i=0; i< len; i++)
        max = GSL_MAX(max, strlen(names[i]));
    return max;
}

//On screen, display a pipe, else use the usual output delimiter.
static void a_pipe(FILE *f, char displaytype){
    if (displaytype == 's')
        fprintf(f, " | ");
    else
        fprintf(f, apop_opts.output_delimiter);

}

static void apop_data_show_core(const apop_data *data, FILE *f, char displaytype){
    if (!data){
        fprintf(f, "NULL\n");
        return;
    }
  int     i, j, L = 0, Lc = 6,
          start   = (data->vector)? -1 : 0,
          end     = (data->matrix)? data->matrix->size2 : 0,
          rowend  = (data->matrix)? data->matrix->size1 : (data->vector) ? data->vector->size : data->text ? data->textsize[0] : -1;
  double  datapt;
    if (data->names->title)
        fprintf(f, "\t%s\n\n", data->names->title);
    if (data->names->rowct)
        L   = get_max_strlen(data->names->row, data->names->rowct);
    if (data->names->rowct)
        fprintf(f, "%*s  ", L+2, " ");
    if (data->vector && data->names->vector){
        fprintf(f, "%*s", L+2, data->names->vector);
    }
    if (data->matrix){
        if (data->vector && data->names->colct){
            fprintf(f, "%c ", data->names->vector ? ' ' : '\t' );
            a_pipe(f, displaytype);
        }
        for(i=0; i< data->names->colct; i++){
            if (i < data->names->colct -1)
                fprintf(f, "%s%s", data->names->column[i], apop_opts.output_delimiter);
            else
                fprintf(f, "%s", data->names->column[i]);
        }
    }
    if (data->textsize[1] && data->names->textct){
        if ((data->vector && data->names->vector) || (data->matrix && data->names->colct))
            a_pipe(f, displaytype);
        for(i=0; i< data->names->textct; i++){
            if (i < data->names->textct -1)
                fprintf(f, "%s%s", data->names->text[i], apop_opts.output_delimiter);
            else
                fprintf(f, "%s", data->names->text[i]);
        }
    }
    fprintf(f, "\n");
    for(j=0; j< rowend; j++){
        if (data->names->rowct > j)
            fprintf(f, "%*s%s", L+2, data->names->row[j], apop_opts.output_delimiter);
        for(i=start; i< end; i++){
            if (i==-1 && data->names->vector) 
                Lc  =  strlen(data->names->vector);
            else if (i>=0 && data->names->colct > i) 
                Lc  =  strlen(data->names->column[i]);
            else
                Lc  =  6;
            if ((i < 0 && j < data->vector->size) || (i> 0 && j < data->matrix->size1 && i < data->matrix->size2)){
                datapt  = apop_data_get(data, j, i);
                if (datapt == (int) datapt)
                    //fprintf(f, "%*i", Lc, (int) datapt);
                    fprintf(f, "%i", (int) datapt);
                else
                    fprintf(f, "%g", datapt);
                //    fprintf(f, "%*f", Lc, datapt);
            } else
                fprintf(f, " ");
            if (i==-1 && data->matrix) 
                a_pipe(f, displaytype);
            if (i < end-1)
                fprintf(f, apop_opts.output_delimiter);
        }
        if (data->text){
            if (data->vector || data->matrix)
                a_pipe(f, displaytype);
            for(i=0; i< data->textsize[1]; i++)
                fprintf(f, "%s%s", data->text[j][i], apop_opts.output_delimiter);
        }
        fprintf(f, "\n");
    }
}


/** Print an \ref apop_data set to a file, the database, or the screen,
  as determined by the \ref apop_opts_type "apop_opts.output_delimiter".
    
\ingroup apop_print */
void apop_data_print(apop_data *data, char *file){
    if (apop_opts.output_type   == 'd'){
        apop_data_to_db(data,  apop_strip_dots(apop_strip_dots(file,1),0));
        return;
    }
    if (apop_opts.output_type   == 'p'){
        apop_data_show_core(data,  apop_opts.output_pipe, 'p');
        return;
    }
    if (file){
        FILE *f = fopen(file, apop_opts.output_append ? "a" : "w");
        apop_data_show_core(data, f, 'p');
        fclose(f);
    } else
        apop_data_show_core(data, stdout, 'p');
}


/* the next function plots a single graph for the \ref apop_plot_lattice  fn */
static void printone(FILE *f, double width, double height, double margin, int xposn, int yposn, const apop_data *d){
    //pull two columns
  size_t count    = d->matrix->size2;
  double nudge    = 0.08;
  gsl_vector  v1  = gsl_matrix_column(d->matrix, xposn).vector;
  gsl_vector  v2  = gsl_matrix_column(d->matrix, yposn).vector;
  gsl_matrix  *m  = gsl_matrix_alloc(d->matrix->size1, 2);
    gsl_matrix_set_col(m, 0, &v1);
    gsl_matrix_set_col(m, 1, &v2);
  double sizex        = (double)(width - margin * (count -1))/count;
  double sizey        = (double)(height - margin * (count -1))/count;
  double offx        = width - (sizex +margin-nudge)* (count - xposn) - nudge*count;
  double offy        = height - (sizey +margin-nudge)* (1 + yposn) - nudge*count;
        fprintf(f, "unset y2tics; unset y2label; unset ytics; unset ylabel\n");
        fprintf(f, "unset x2tics; unset x2label; unset xtics; unset xlabel\n");
    fprintf(f, "set size   %g, %g\n\
set origin %g, %g\n\
#plot '-'        \n\
            ",  sizex, sizey, offx, offy);
    //Gnuplot has no way to just set a label with no plot, so 
    //labels have to be drawn with a long offset from one of the
    //plots we do display.
    if (xposn  == yposn+1)
        fprintf(f, "set label %i '%s' center at graph %g, %g\n",yposn+1, (d->names->colct >yposn)? d->names->column[yposn]: "",  -0.5, 0.5);
    if ((yposn == count-1) && (xposn  == count-2))
        fprintf(f, "set label %i '%s' center at graph %g, %g\n",count, (d->names->colct >count -1)? d->names->column[count -1]: "",  1.5, 0.5);
    if (xposn != yposn){
        fprintf(f, "plot '-'\n");
        fflush(f);
        char tmptype = apop_opts.output_type;
        FILE *tp     = apop_opts.output_pipe;
        apop_opts.output_type = 'p';
        apop_opts.output_pipe = f;
        apop_matrix_print(m, NULL);
        apop_opts.output_type = tmptype;
        apop_opts.output_pipe = tp;
        fprintf(f,"e\n");
        gsl_matrix_free(m);
    } 
    if (xposn  == yposn+1)
        fprintf(f, "unset label %i \n",yposn+1);
    if ((yposn == count-1) && (xposn  == count-2))
        fprintf(f, "unset label %i\n",count);
}

/*
static void printlabel(char filename[], char *name){
    //maybe some day this will have content.
}
*/

/** This produces a Gnuplot file that will produce an array of 2-D
 plots, one for each pair of columns in the data set. Along the diagonal
 is a plot of the variable against itself---a density plot of the variable.

 \param d       The data set whose (matrix) columns will be compared. (No default, must not be \c NULL.)
 \param outfile The output file, to which a Gnuplot command file will be written. If \c NULL, print to STDOUT (default = \c NULL.)

\image latex "lattice.png" "A lattice showing three variables graphed against each other."
\image html "lattice.png" "A lattice showing three variables graphed against each other."

This function uses the \ref designated syntax for inputs.

\ingroup output
 */
#ifdef APOP_NO_VARIADIC 
 void apop_plot_lattice(const apop_data *d, char *outfile){
#else
apop_varad_head( void , apop_plot_lattice){
    const apop_data * apop_varad_var(d, NULL);
    apop_assert_void(d, 0, 's', "Input data set is NULL.\n");
    char * apop_varad_var(outfile, NULL);
    apop_plot_lattice_base(d, outfile);
}

 void apop_plot_lattice_base(const apop_data *d, char *outfile){
#endif 
  double  width   = 1.2,//these used to be options, but who's ever gonna set them to something else.
          height  = 1.2;
  double  margin  = 0;
  FILE    *f;
  int     i,j;
    if (apop_opts.output_type == 'f'){
        if (outfile)
            f      = fopen(outfile, "a");
        else
            f      = stdout;
    } else if (apop_opts.output_type == 'p')
        f      = apop_opts.output_pipe;
    else if (apop_opts.output_type == 's')
        f      = stdout;
    else {
            apop_error(0, 'c', "apop_plot_lattice: please set apop_opts.output_type = 'f', 'p', or 's' before using this function.\n");
            return;
        }
    fprintf(f, "set size %g, %g\n\
set rmargin 5\n\
set lmargin -1\n\
set tmargin 2.4\n\
set bmargin -2\n\
set origin %g, %g       \n\
set multiplot   #layout %i, %i downwards        \n\
unset xtics; unset xlabel; unset ytics; unset ylabel\n\
set nokey           \n\
        ", width, height, margin,margin, d->matrix->size2, d->matrix->size2);
    for (i = 0; i< d->matrix->size2; i++)
        for (j = 0; j< d->matrix->size2; j++)
            printone(f, width, height, margin, i, j, d);
    fprintf(f, "unset multiplot\n"); 
    if (apop_opts.output_type == 'f' && outfile)
        fclose(f);
}

/** Plot the percentiles of a data set against the percentiles of a distribution.

The distribution percentiles will be on the $x$-axis, your data percentiles on the $y$-.

The function respects the <tt>output_type</tt> option.
It uses the \ref designated syntax for inputs.

\param v    The data (No default, must not be \c NULL.)
\param m    The distribution, such as apop_normal. I'll be using the \c draw method. (Default = best-fitting Normal)
\param outfile   The name of the text file to print to.  If NULL then write to STDOUT. (Default = \c NULL)
\param bins The number of bins in the histogram. The number of points on the plot will always be 101 (i.e. percentiles). (default = MAX(10, data->size/10); denominator subject to future adjustment)
\param r    A \c gsl_rng. If NULL, I'll take care of the RNG; see \ref autorng. (Default = \c NULL)
*/
#ifdef APOP_NO_VARIADIC 
 void apop_plot_qq(gsl_vector *v, apop_model *m, char *outfile, size_t bins, gsl_rng *r){
#else
apop_varad_head( void , apop_plot_qq){
    static gsl_rng *spare = NULL;
    int free_m = 0;
    gsl_vector * apop_varad_var(v, NULL);
    apop_assert_void(v, 0, 's', "Input vector is NULL.\n");
    apop_model  *apop_varad_var(m, NULL);
    if (!m){
        free_m++;
        apop_data *d = apop_vector_to_data(v);
        m = apop_estimate(d, apop_normal);
        d->vector = NULL;
        apop_data_free(d);
    }
    char * apop_varad_var(outfile, NULL);
    size_t apop_varad_var(bins, GSL_MAX(10, v->size/10));
    gsl_rng *apop_varad_var(r, NULL)
    if (!r && !spare) 
        spare = apop_rng_alloc(++apop_opts.rng_seed);
    if (!r)  r = spare;

    apop_plot_qq_base(v, m, outfile, bins, r);
    if (free_m) apop_model_free(m);
}

 void apop_plot_qq_base(gsl_vector *v, apop_model *m, char *outfile, size_t bins, gsl_rng *r){
#endif
  FILE  *f;
  double *pctdata = apop_vector_percentiles(v, 'a');

    //produce percentiles from the model via RNG.
  gsl_vector  *vd  = gsl_vector_alloc(bins);
  int         i;
    for(i=0; i< bins; i++)
        m->draw(gsl_vector_ptr(vd, i), r, m);
    double *pctdist = apop_vector_percentiles(vd, 'a');

    if (apop_opts.output_type == 'p')
        f   = apop_opts.output_pipe;
    else 
        f   = (outfile ? fopen(outfile, "a") : stdout);
    fprintf(f, "set key off; set size square;\n\
plot x;\n\
replot '-' with points\n");
    //I can't tell which versions of Gnuplot support this form:
    /*fprintf(f, "set key off                                       ;\n\
                        set style data histograms               ;\n\
                        set style histogram cluster gap 0       ;\n\
                        set xrange [0:%i]                       ;\n\
                        set style fill solid border -1          ;\n\
                        set boxwidth 0.9                        ;\n\
                        plot '-' using 2:xticlabels(1);\n", bins);*/
    for (i=0; i < 101; i++)
        fprintf(f, "%g\t %g\n", pctdist[i], pctdata[i]);
    fprintf(f, "e\n");
    if (apop_opts.output_type == 'p')
        fflush(f);
    else if (outfile)
        fclose(f);
    gsl_vector_free(vd);
    gsl_rng_free(r);
}
